/*----------------------------------------------------------------------
* File    : item.cpp
* Descr.  : itemset management where items are attribute-value pairs
*           (including missing values)
* Author  : Michael Mampaey
* Date    : 01/03/2006
* Note    : dirty tricks inhere: mutable fields to be able to change
*           them when the object is key in an associative container
*           but the field isn't considered by operators < or ==
* --------------------------------------------------------------------*/

#include "item.h"
using namespace std;


Value::Value(string *v, int s) : support(s)
{
    transactions = new vector<int>();
    setVal(v);
    ++instanceCount;
}


bool Value::setVal(string *v)
{
    if (v) {
        val = *((inserter=values.insert(v)).first);
        return inserter.second;
    }
    else
        val = 0;
    return false;
}


void Value::cleanup()
{
    set<string*, StringPtrCmp>::iterator it=values.begin();
    while (it!=values.end()) delete *it++;
    values.clear();
}


bool Value::operator< (const Value &i) const
{
    if (val && i.val)
        return ((*val).compare(*(i.val))<0);
    else
        return val<i.val;
}


bool Value::operator!= (const Value &i) const
{
    if (val && i.val)
        return *val!=*(i.val);
    else
        return val != i.val;
}

bool Value::operator== (const Value &i) const
{
    if (val && i.val)
        return *val==*(i.val);
    else
        return val == i.val;
}


set<string*, StringPtrCmp> Value::values;
string *Value::star = new string("");
string *Value::starToken = new string("*");
string *Value::questToken = new string("?");
pair<set<std::string*, StringPtrCmp>::iterator, bool> Value::inserter;
int Value::instanceCount = 0;



Attribute::Attribute() : attr(0), mXminsup(0.0)
{
    values = new set<Value>();
    attrItem = new Value(Value::star, 0);
    reset();
}


Attribute::Attribute(int a) : attr(a), mXminsup(0.0)
{
    values = new set<Value>();
    attrItem = new Value(Value::star, 0);
    reset();
}


Attribute::Attribute(const Attribute& a) : 
            attr(a.attr), mXminsup(a.mXminsup)
{
    values = new set<Value>(*(a.values));
    if (a.current==a.values->end())
        current = values->end();
    else
        current = values->find(*(a.current));
    attrItem = new Value(*(a.attrItem));
}


set<Value>::iterator Attribute::addValue(const Value& v) const
{
    return values->insert(v).first; // mind behaviour for transactions
}

void Attribute::setAttrItem(const Value& s)
{
    attrItem->support = s.support;
    attrItem->transactions = s.transactions;
    return;
}

void Attribute::stealAttrItem(const Attribute& a) const
{
    delete attrItem;
    attrItem = a.attrItem;
    a.attrItem = 0;
    return;
}
