/*----------------------------------------------------------------------
* File    : item.h
* Descr.  : Itemset management, where items are attribute-value pairs
*           (including missing values)
* Author  : Michael Mampaey
* Date    : 01/03/2005
* Note    : dirty tricks inhere: mutable fields to be able to change
*           them when the object is key in an associative container
*           but the field isn't considered by operators < or ==
* --------------------------------------------------------------------*/

#include <vector>
#include <string>
#include <set>
using namespace std;



struct StringPtrCmp
{
    bool operator()(const string *a, const string *b) const
    { return *a < *b; }
};



/* Values belong to an attribute, and occur in some transactions 
 * null pointer denotes missing value ('?')
 * "" denotes non-missing value ('*')
 * other values stand for themselves
 * special values must be evaluated before actual attribute values
 */
class Value
{
public:
    Value(string *v, int s = 0);
    Value(const Value& v) : val(v.val), support(v.support)
    {transactions = new vector<int>(*(v.transactions));  ++instanceCount; }
    ~Value() { delete transactions; if (--instanceCount==0) cleanup(); }
    
    bool setVal(string *v);
    
    inline bool operator< (const Value &i) const;
    inline bool operator!=(const Value &i) const;
    inline bool operator==(const Value &i) const;
    
    string *val;
    mutable int support;
    mutable vector<int> *transactions;
    
    static string* star;
    static string* starToken;
    static string* questToken;
    
private:
    
    static set<string*, StringPtrCmp> values;
    static pair<set<string*, StringPtrCmp>::iterator, bool> inserter;
    static int instanceCount;
    static void cleanup();
};


/* Attributes have values, and can be missing (?) or not missing (*)
 * an internal iterator points to the value currently being considered
 * */
class Attribute
{
public:
    Attribute();
    Attribute(int a);
    Attribute(const Attribute& a);
    ~Attribute() {delete values; delete attrItem;}
    
    const Value& getCurrent() const {return *current;}
    void nextValue() const {current++;}
    bool currentIsValid() const {return current!=values->end();}
    void reset() const {current = values->begin();}
    
    set<Value>::iterator addValue(const Value& v) const;
    void setAttrItem(const Value& s);
    void stealAttrItem(const Attribute& a) const;
    double mXminsup; // for itemset X, this approixmates m(X)*minsup
    
    bool operator< (const Attribute& a) const {return attrItem->support<a.attrItem->support;}
    
    int attr;
    set<Value> *values;
    mutable Value *attrItem;
    
private:
    mutable set<Value>::iterator current;
};
